;--------------------------------------------------------------------
; Proc name : Soundex.a()
; Syntax    : Soundex.a( string.a )
; Purpose   : Returns a soundex code for the specified string
;             Case of the string does not matter. The soundex routine 
;             always converts it uppercase.
; Returns   : string of a soundex code 4 chars long
; Example   :
;     Table with the following fields exist :
;         [LastName]  Ax          ; x any numeric
;         [Soundex]   A4          ; A999   - that is a Character with 3 digits
;
;  1.   [Soundex] = Soundex.a( [LastName] )
;
;  2.   Lastname.a= "SomeName"
;       soundexcode.a = Soundex.a( Lastname.a )
;
; Overview
;
;   In brief, this soundex algorithm is the "standard" soundex 
;   method (several variations exist). It parses a string for 
;   the numeric (0-6) equivalent of the strings individual 
;   characters. The first character of the string plus the 
;   first 3 non-zero non-consecutive, numeric equivalents form 
;   the complete soundex equivalent. 
;
;   This code was ported from a C program available on CompuServ
;   BPROGB/TURBO C 1.x/SOUNDE.C
;
;
; I have this code working in my applications. I have also created a generic
; search routine that will allow the user to enter in a LastName string
; and select the type of search (Exact/Bestmatch/Soundex) but that is left
; as an excercise for the reader.
;----------------------------------------------------------------------------
; Shareware/Copyright ???  This is pure code ! I didn't design the SOUNDEX
; fuctions, I just made it useable in PAL code. Use it as you wish
; want I don't care.
;  -> If it works for you then you may use it.
;  -> If it doesn't work for then you have two choices (1) Don't use it
;     or (2) Make it work.
;
; If it works for you, a simple compuserv "Thanks" will do.
;
;  Mike Wendling
;  Compuserv 76247,142
;
PROC Soundex.a( AnyString )
  ; Returns the soundex string for a given string

  Private RtnString,CurrentCode,CurrentChar,LastCode,SoundexPntr,StrPosPntr

  IF RTRIM( AnyString ) = "" THEN
     RETURN ""                                ; Name was Blank
  ENDIF

  IF SUBSTR(TYPE(AnyString),1,1) <> "A" THEN  ; Accept only Strings
     RETURN ""                                ; Error out
  ENDIF

  CurrentChar = Upper(Substr(AnyString,1,1))
  RtnString = CurrentChar+"000"         ;Init ret value to 1st char + "000"

  LastCode  = Soundex.GetScanCode.a(CurrentChar)  ;Store first chars Soundex
                                        ;code for double letter
                                        ;checking at beginning of string

  SoundexPntr  = 2                      ;Initialize position pointers
  StrPosPntr   = 2

  ; Scan through length of anystring or until soundexstring is full
  While (Len(AnyString) >= StrPosPntr) And SoundexPntr <= 4 
     CurrentChar = Substr(AnyString,StrPosPntr,1)     ; isolate char to process
     CurrentCode = Soundex.GetScanCode.a(CurrentChar) ; get its soundex code

     If (CurrentCode <> "0") And (CurrentCode <> LastCode) then ;Insert scan code if it is different
                                                                ;from previous code and not "0"
        RtnString = Substr(RtnString,1,SoundexPntr-1) + CurrentCode +
                    Substr(RtnString,SoundexPntr+1,Len(RtnString))
        LastCode  = CurrentCode
        SoundexPntr = SoundexPntr + 1
     Endif
     StrPosPntr = StrPosPntr + 1
  EndWhile
  RETURN RtnString
ENDPROC

;--------------------------------------------------------------------
; Proc name : Soundex.GetScanCode.a()
; Syntax    : Soundex.GetScanCode.a( char.a )
; Purpose   : Returns a soundex scan code for the specified character
; Returns   : char
; comments  : Used in Soundex.a() proc
PROC Soundex.GetScanCode.a( AnyAlphaChar )
  ;returns the indexed soundex scan code for a given char
  Private SoundexMap,CodePointer

               ; ABCDEFGHIJKLMNOPQRSTUVWXYZ ;
               ; :::::::::::::::::::::::::: ;
  SoundexMap =  "01230120022455012623010202"

  CodePointer = (Asc(Upper(AnyAlphaChar))-Asc("A")+1)

  IF CodePointer < 0 or CodePointer > 26 then  ;If not Alpha char
     RETURN "0"                                ; return "0" i.e. ignore
  ELSE
     RETURN Substr(SoundexMap,CodePointer,1)   ; return Soundex value
  ENDIF

ENDPROC

str = "Smith"
c = Soundex.a(str)
Message "The soundex code for "+str+" is "+c
n=getchar()

str = "Smyth"
c = Soundex.a(str)
Message "The soundex code for "+str+" is "+c
n=getchar()

str = "Woodman"
c = Soundex.a(str)
Message "The soundex code for "+str+" is "+c
n=getchar()

str = "WOODMEER"
c = Soundex.a(str)
Message "The soundex code for "+str+" is "+c
n=getchar()

